# Py Data Stack
import numpy as np

# Neural Network
import torch
import torch.nn as nn
import torch.nn.functional as F


class AvaxNN(nn.Module):

	def __init__(self, num_features, learning_rate=1e-4, optimizer=torch.optim.AdamW, loss_fn=nn.BCELoss(), device='cpu'):
		super(AvaxNN, self).__init__()
		self.fc1 = nn.Linear(num_features, num_features//2)
		self.fc2 = nn.Linear(num_features//2, num_features//4)
		self.fc3 = nn.Linear(num_features//4, 2)
		self.dropout1 = nn.Dropout(0.40)
		self.dropout2 = nn.Dropout(0.40)

		self.device = device
		self.optimizer = optimizer(self.parameters(), lr=learning_rate)
		self.loss_fn = loss_fn.to(self.device)

	# x represents our data
	def forward(self, x):
		x = self.fc1(x)
		x = torch.tanh(x) 
		x = self.dropout1(x)

		x = self.fc2(x)
		x = torch.tanh(x) 
		x = self.dropout2(x)

		x = self.fc3(x)
		output = F.softmax(x, dim=1)
		return output

	def fit(self, train_loader, valid_loader, epochs=1000):
		for epoch in range(epochs):
			self.train() 

			LOSS_train = 0.0
			PRECISION_0_train, RECALL_0_train, F1_0_train, ACCURACY_0_train = 0.0, 0.0, 0.0, 0.0
			PRECISION_1_train, RECALL_1_train, F1_1_train, ACCURACY_1_train = 0.0, 0.0, 0.0, 0.0
			
			for j, data in enumerate(train_loader, 0):

				# Get the inputs; data is a list of [inputs, labels]
				inputs, labels = data
				inputs = inputs.to(self.device)
				labels = labels.to(self.device)

				# Zero the parameter gradients
				self.optimizer.zero_grad()

				# Forward (aka predict)
				outputs = self.forward(inputs)
				predictions = np.array([[1.0, 0.0] if np.argmax(p.cpu().detach().numpy())==0 else [0.0, 1.0] for p in outputs])

				# Loss + Backprop
				loss = self.loss_fn(outputs, labels)
				LOSS_train += loss.item()
				loss.backward()
				self.optimizer.step()

				# Metrics
				(precision, recall, f1, accuracy) = self.get_metrics(predictions, labels, for_class=0)
				PRECISION_0_train += precision
				RECALL_0_train += recall
				F1_0_train += f1
				ACCURACY_0_train += accuracy
				
				(precision, recall, f1, accuracy) = self.get_metrics(predictions, labels, for_class=1)
				PRECISION_1_train += precision
				RECALL_1_train += recall
				F1_1_train += f1
				ACCURACY_1_train += accuracy    

			# Average the metrics based on the number of batches
			LOSS_train /= len(train_loader)
			PRECISION_0_train /= len(train_loader)
			RECALL_0_train /= len(train_loader)
			F1_0_train /= len(train_loader)
			ACCURACY_0_train /= len(train_loader)
			PRECISION_1_train /= len(train_loader)
			RECALL_1_train /= len(train_loader)
			F1_1_train /= len(train_loader)
			ACCURACY_1_train /= len(train_loader)

			# Validation set
			self.eval()
			with torch.no_grad():

				LOSS_valid = 0.0
				PRECISION_0_valid, RECALL_0_valid, F1_0_valid, ACCURACY_0_valid = 0.0, 0.0, 0.0, 0.0
				PRECISION_1_valid, RECALL_1_valid, F1_1_valid, ACCURACY_1_valid = 0.0, 0.0, 0.0, 0.0
				
				for k, data in enumerate(valid_loader, 0):

					# Get the inputs; data is a list of [inputs, labels]
					inputs, labels = data
					inputs = inputs.to(self.device)
					labels = labels.to(self.device)

					# Forward (aka predict)
					outputs = self.forward(inputs)
					predictions = np.array([[1.0, 0.0] if np.argmax(p.cpu().detach().numpy())==0 else [0.0, 1.0] for p in outputs])

					# Loss
					loss = self.loss_fn(outputs, labels)
					LOSS_valid += loss.item()

					# Metrics
					(precision, recall, f1, accuracy) = self.get_metrics(predictions, labels, for_class=0)
					PRECISION_0_valid += precision
					RECALL_0_valid += recall
					F1_0_valid += f1
					ACCURACY_0_valid += accuracy
					
					(precision, recall, f1, accuracy) = self.get_metrics(predictions, labels, for_class=1)
					PRECISION_1_valid += precision
					RECALL_1_valid += recall
					F1_1_valid += f1
					ACCURACY_1_valid += accuracy

				# Average the metrics based on the number of batches
				LOSS_valid /= len(valid_loader)
				PRECISION_0_valid /= len(valid_loader)
				RECALL_0_valid /= len(valid_loader)
				F1_0_valid /= len(valid_loader)
				ACCURACY_0_valid /= len(valid_loader)
				PRECISION_1_valid /= len(valid_loader)
				RECALL_1_valid /= len(valid_loader)
				F1_1_valid /= len(valid_loader)
				ACCURACY_1_valid /= len(valid_loader)


			##################
			### STATISTICS ###
			##################

			if (epoch+1)%50 == 0:
				print(f'Epoch {epoch+1}/{epochs}'+'\n')
				print(f'Train Loss: {LOSS_train:.5f}  |  Valid Loss: {LOSS_valid:.5f}')
				print(f'Train Accu: {ACCURACY_0_train:.5f}  |  Valid Accu: {ACCURACY_0_valid:.5f}')
				print()
				print("Class 0:")
				print(f'Train Prec: {PRECISION_0_train:.5f}  |  Valid Prec: {PRECISION_0_valid:.5f}')
				print(f'Train Rcll: {RECALL_0_train:.5f}  |  Valid Rcll: {RECALL_0_valid:.5f}')
				print(f'Train  F1 : {F1_0_train:.5f}  |  Valid  F1 : {F1_0_valid:.5f}')
				print()
				print("Class 1:")
				print(f'Train Prec: {PRECISION_1_train:.5f}  |  Valid Prec: {PRECISION_1_valid:.5f}')
				print(f'Train Rcll: {RECALL_1_train:.5f}  |  Valid Rcll: {RECALL_1_valid:.5f}')
				print(f'Train  F1 : {F1_1_train:.5f}  |  Valid  F1 : {F1_1_valid:.5f}')
				print()
				print('-'*43)
				print()
		
		print('Finished Training')


	def get_metrics(self, preds, labels, for_class=1):
		TP, FP, TN, FN = 0, 0, 0, 0

		# Iterate over all predictions
		for idx in range(len(preds)):
			# If we predicted the sample to be class {for_class}
			if preds[idx][for_class] == 1:
				# Then check whether the prediction was right or wrong
				if labels[idx][for_class] == 1:
					TP += 1
				else:
					FP += 1
			# Else we predicted another class 
			else:
				# Check whether the "not class {for_class}" prediction was right or wrong
				if labels[idx][for_class] != 1:
					TN += 1
				else:
					FN += 1

		precision = TP/(TP+FP) if TP+FP > 0 else 0 # Of all "class X" calls I made, how many were right?
		recall = TP/(TP+FN) if TP+FN > 0 else 0 # Of all "class X" calls I should have made, how many did I actually make?
		f1 = (2*precision*recall)/(precision+recall) if precision+recall > 0 else 0
		accuracy = (TP+TN)/(TP+FP+TN+FN)

		return (precision, recall, f1, accuracy)


	def predict(self, X_input):
		self.eval()
		result = []
		
		# Predict from PyTorch dataloader
		if type(X_input) == torch.utils.data.dataloader.DataLoader:
			
			with torch.no_grad():
				for k, data in enumerate(X_input, 0):
					# Get the inputs; data is a list of [inputs, labels]
					inputs, labels = data
					inputs = inputs.to(self.device)
					labels = labels.to(self.device)
					# Forward (aka predict)
					outputs = self.forward(inputs)
					predictions = np.array([[1.0, 0.0] if np.argmax(p.cpu().detach().numpy())==0 else [0.0, 1.0] for p in outputs])

					if len(result)==0:
						result = predictions
					else:
						result = np.concatenate((result, predictions))

				return np.array([np.argmax(x) for x in result])
		
		# Predict from Numpy array or list
		else:
			
			if type(X_input) == list:
				X_input = np.array(X_input)
		
			if isinstance(X_input, np.ndarray): 
				X_test = torch.from_numpy(X_input).float()

				with torch.no_grad():
					for k, data in enumerate(X_test):
						# Get the inputs; data is a list of [inputs] 
						inputs = data.reshape(1,-1).to(self.device)
						# Forward (aka predict)
						outputs = self.forward(inputs)
						predictions = np.array([[1.0, 0.0] if np.argmax(p.cpu().detach().numpy())==0 else [0.0, 1.0] for p in outputs])

						if len(result)==0:
							result = predictions
						else:
							result = np.concatenate((result, predictions))

					return np.array([np.argmax(x) for x in result])
						  
			else:
				raise("Input must be a dataloader, numpy array or list")


	def predict_proba(self, X_input):
		self.eval()
		result = []
		
		# Predict from PyTorch dataloader
		if type(X_input) == torch.utils.data.dataloader.DataLoader:

			with torch.no_grad():
				for k, data in enumerate(X_input, 0):
					# Get the inputs; data is a list of [inputs, labels]
					inputs, labels = data
					inputs = inputs.to(self.device)
					labels = labels.to(self.device)
					# Forward (aka predict)
					outputs = self.forward(inputs)
					class_proba = outputs.cpu().detach().numpy()
					
					if len(result)==0:
						result = class_proba
					else:
						result = np.concatenate((result, class_proba))

				return result
		
		# Predict from Numpy array or list
		else:
			
			if type(X_input) == list:
				X_input = np.array(X_input)
		
			if isinstance(X_input, np.ndarray):  
				X_input = torch.from_numpy(X_input).float()

				with torch.no_grad():
					for k, data in enumerate(X_input):
						# Get the inputs; data is a list of [inputs] 
						inputs = data.reshape(1,-1).to(self.device)
						# Forward (aka predict)
						outputs = self.forward(inputs)
						class_proba = outputs.cpu().detach().numpy()

						if len(result)==0:
							result = class_proba
						else:
							result = np.concatenate((result, class_proba))

					return result

			else:
				raise("Input must be a dataloader, numpy array or list")




from AvaxModel import AvaxModel

consumer_key = ''
consumer_secret = ''
access_token = ''
access_secret = ''
bearer_token = ''


if __name__ == "__main__":

	# Get the userid
	userid = '369583954'

	# Predict
	model = AvaxModel(consumer_key, consumer_secret, access_token, access_secret, bearer_token)
	pred_proba = model.predict_from_userid_api_v1(userid)

	# Results
	print(f'User: {userid}')
	print(f'Class Probabilities: {pred_proba}')


# Custom Modules
import Avax_NN
import DF_from_DICT

# Py Data Stack
import numpy as np
import pandas as pd

# Neural Network
import torch
import torch.nn as nn
import torch.nn.functional as F

# Feature Engineering
import sent2vec

# File Manipulation
from glob import glob
import joblib

# Feature Scaling
from sklearn.preprocessing import RobustScaler

# Twitter 
import tweepy
import requests

# NLP
from nltk.tokenize import TweetTokenizer
from nltk.corpus import stopwords



class AvaxModel():
    
    def __init__(self, consumer_key=None, consumer_secret=None, access_token=None, access_secret=None, bearer_token=None):
        super(AvaxModel, self).__init__()
        
        # Connect to Twitter
        self.consumer_key = consumer_key
        self.consumer_secret = consumer_secret
        self.access_token = access_token
        self.access_secret = access_secret

        self.api_v1_connection = False
        if consumer_key and consumer_secret and access_token and access_secret:
            self.auth = tweepy.OAuthHandler(consumer_key, consumer_secret)
            self.auth.set_access_token(access_token,access_secret)
            self.api = tweepy.API(self.auth, retry_count=5, retry_delay=2, wait_on_rate_limit=True, wait_on_rate_limit_notify=True)
            self.api_v1_connection = True

        self.api_v2_connection = False
        if bearer_token:
            self.headers = {"Authorization": f"Bearer {bearer_token}"}
            self.api_v2_connection = True

        # Load sent2vec model
        self.sent2vec_model = sent2vec.Sent2vecModel()
        self.sent2vec_model.load_model('wiki_unigrams.bin')#, inference_mode=True)

        # Load trained scaler
        self.scaler = joblib.load('scaler1.joblib') 

        # Tokenizer
        #self.tknzr = TweetTokenizer(preserve_case=False, reduce_len=False, strip_handles=False)
        self.stopW = stopwords.words('english')
        self.stopW.append("https://t.co")
        
    def predict_from_userid_api_v1(self, userid):
        if self.api_v1_connection:
            # Given a user ID, crawl its last 3000 tweets as a list of dictionaries
            user_timeline = [status._json for status in tweepy.Cursor(self.api.user_timeline, id=userid).items(100)]
            print(f'User: {userid}. Timeline length: {len(user_timeline)}')

            # Extract all the features from the list of dictionaries and convert it to a datadframe
            df = DF_from_DICT.main(user_timeline) 

            # Generate timeseries features based on a user's tweets
            #timeseries_features = Feature_Engineering.prediction_timeseries_generator(df, self.scaler, sent2vec_model=self.sent2vec_model)
            timeseries_features = self.prediction_timeseries_generator_text_only(df.token.to_list(), self.scaler, sent2vec_model=self.sent2vec_model)

            # Setting the device
            device = torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')

            # Load trained model
            model = Avax_NN.AvaxNN(num_features=timeseries_features.shape[1], 
                                   learning_rate=1e-4, 
                                   optimizer=torch.optim.AdamW, 
                                   loss_fn=nn.BCELoss(), 
                                   device=device)
            model.load_state_dict(torch.load('model_pytorch1.pt', map_location=torch.device(device)))
            model.eval()

            # Send model to the device
            model = model.to(device)

            # Predict
            pred_proba = model.predict_proba(timeseries_features)
            return pred_proba

        else:
            print("In order to predict from a user id you need to input your twitter credentials to the class constructor. Please pass 'consumer_key', 'consumer_secret', 'access_token', 'access_secret'.")
    
    def predict_from_userid_api_v2(self, userid):
        if self.api_v2_connection:
            # Df to store user tweets 
            df_all = pd.DataFrame()
            
            # Api v2 only allows 100 tweets per call, so we need to paginate to obtain more tweets
            pagination_token = None
            while df_all.shape[0] < 100:
                response = self.make_request(self.headers, userid, pagination_token)       
                if 'data' not in response:
                    print(response)
                df = pd.DataFrame(response['data'])
                df_all = pd.concat((df_all, df))
                try:
                    pagination_token = response['meta']['next_token']
                except:
                    break

            list_of_tweets = df_all['text'].to_list()
            print(f'User: {userid}. Timeline length: {len(list_of_tweets)}')

            pred_proba = self.predict_from_tweets(list_of_tweets)
            return pred_proba

        else:
            print("In order to predict from a user id you need to input your twitter credentials to the class constructor. Please pass 'bearer_token'.")

    def predict_from_tweets(self, list_of_tweets):
        # Ensure tweets are tokenized
        #list_of_tweets = [' '.join(self.tknzr.tokenize(str(tweet))) for tweet in list_of_tweets]
        list_of_tweets = [' '.join([word for word in tweet.split() if word not in self.stopW and "https://t.co" not in word]) for tweet in list_of_tweets]
        list_of_tweets = [self.clean_tweet(tweet) for tweet in list_of_tweets]

        # Generate timeseries features based on a user's tweets
        timeseries_features = self.prediction_timeseries_generator_text_only(list_of_tweets, self.scaler, sent2vec_model=self.sent2vec_model)

        # Setting the device
        device = torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')

        # Load trained model
        model = Avax_NN.AvaxNN(num_features=timeseries_features.shape[1], 
                               learning_rate=1e-4, 
                               optimizer=torch.optim.AdamW, 
                               loss_fn=nn.BCELoss(), 
                               device=device)
        model.load_state_dict(torch.load('model_pytorch1.pt', map_location=torch.device(device)))
        model.eval()

        # Send model to the device
        model = model.to(device)

        # Predict
        pred_proba = model.predict_proba(timeseries_features)
        return pred_proba
        
    def prediction_timeseries_generator_text_only(self, list_of_tweets, scaler, sent2vec_model=None):
        # Concatenate all of a users tweets in that period, then embed them with sent2vec twitter_bigrams model
        user_tweets = ' '.join(str(x) for x in list_of_tweets)
        embedding = sent2vec_model.embed_sentence(user_tweets)

        # Merge text features (embedding), user features, url features and label
        user_data = np.concatenate([embedding[0]])
        timeseries_dataset = np.array(user_data)

        # Rescale features
        timeseries_features = scaler.transform([timeseries_dataset])
        return timeseries_features

    def clean_tweet(self, tokenized_text):
        tokenized_text = str(tokenized_text)
        tokenized_text = tokenized_text.replace("RT", "")
        tokenized_text = tokenized_text.replace("#", "")
        tokenized_text = tokenized_text.replace("&amp;", "")
        tokenized_text = tokenized_text.replace("…", "")
        tokenized_text = tokenized_text.replace (",", " ")
        tokenized_text = tokenized_text.replace("\n", " ")
        tokenized_text = tokenized_text.replace("!", "")
        tokenized_text = tokenized_text.replace(":", "")
        tokenized_text = tokenized_text.replace("(", " ")
        tokenized_text = tokenized_text.replace(")", " ")
        tokenized_text = tokenized_text.replace("/", " ")
        tokenized_text = tokenized_text.replace('"', " ")
        tokenized_text = tokenized_text.replace('?', " ")
        tokenized_text = tokenized_text.lower()
        return tokenized_text

    def make_request(self, headers, user_id, pagination_token=None):
        url = f"https://api.twitter.com/2/users/{user_id}/tweets"
        if pagination_token:
            params = f"max_results=100&tweet.fields=id,text&pagination_token={pagination_token}"
        else:
            params = f"max_results=100&tweet.fields=id,text"
        return requests.request("GET", url, params=params, headers=headers).json()


import pandas as pd
import numpy as np
import pandas as pd
import pdb
from vaderSentiment.vaderSentiment import SentimentIntensityAnalyzer
import nltk
nltk.download('stopwords', quiet=True)
from nltk.corpus import stopwords 
from nltk.tokenize import TweetTokenizer
import sys



def get_files(directory,file_ext):
    file_list = []
    for root,dirs,files in os.walk(directory):
        for file in files:
            if file.endswith(file_ext):
                file_list.append(file)
    file_list.sort()
    return file_list

def get_mentions(status):
    mentionid,mentionsn = [],[]
    if len(status['entities']['user_mentions']) > 0:
        for i in range(len(status['entities']['user_mentions'])):
            mentionid.append(status['entities']['user_mentions'][i]['id'])
            mentionsn.append(status['entities']['user_mentions'][i]['screen_name'])
    return(mentionid,mentionsn)

def get_media_urls(status, tweet_type, extended_text): 
    media_urls = []
    rt_media_urls = []
    q_media_urls = []
    
    try: 
        if "extended_tweet" in status: 
            for url in status["extended_tweet"]["entities"]['media']: 
                media_urls.append(url['media_url'])
        else: 
            urls = status["entities"]["urls"]
            for url in status["entities"]['media']: 
                media_urls.append(url['media_url'])
    except: 
        pass

    try: 
        if 'retweeted_status' in status: 
            if "extended_tweet" in status["retweeted_status"]: 
                for url in status["retweeted_status"]["extended_tweet"]["entities"]['media']: 
                    rt_media_urls.append(url['media_url'])
            else: 
                for url in status["retweeted_status"]["entities"]['media']: 
                    rt_media_urls.append(url['media_url'])
    except: 
        pass

    try: 
        if 'quoted_status' in status: 
            if "extended_tweet" in status['quoted_status']: 
                for url in status["quoted_status"]["extended_tweet"]["entities"]['media']: 
                    q_media_urls.append(url['media_url'])
            else: 
                for url in status["quoted_status"]["entities"]['media']: 
                    q_media_urls.append(url['media_url'])
    except: 
        pass

    return (media_urls, rt_media_urls, q_media_urls)

def get_urls(status, tweet_type, extended_text):

    urls = []
    rt_urls = []
    q_urls = []

    if "extended_tweet" in status: 
        urls = status["extended_tweet"]["entities"]["urls"]
    else: 
        urls = status["entities"]["urls"]

    if 'retweeted_status' in status: 
        if "extended_tweet" in status["retweeted_status"]: 
            rt_urls = status["retweeted_status"]["extended_tweet"]["entities"]["urls"]
        else: 
            rt_urls = status["retweeted_status"]["entities"]["urls"]
    if 'quoted_status' in status: 
        if "extended_tweet" in status['quoted_status']: 
            q_urls = status["quoted_status"]["extended_tweet"]["entities"]['urls']
        else: 
            q_urls = status["quoted_status"]["entities"]["urls"]

    return (urls, rt_urls, q_urls)

def get_rt_urls(status,tweet_type,extended_text):
    #pdb.set_trace()
    # look through entities from the root 
    rt_urls_list = []

    if tweet_type == 'retweeted_tweet_without_comment' and extended_text == 'yes':
        try: 
#           print(count)
            rt_urls_list=status["retweeted_status"]["extended_tweet"]["entities"]["urls"]

        except:
            pass
            
    if tweet_type == 'retweeted_tweet_without_comment' and extended_text == 'no':
        try: 
#           print(count)
            rt_urls_list=status["retweeted_status"]["entities"]["urls"]

        except:
            pass
    return rt_urls_list

def get_tweet_text(status):
    rt_text = ""
    qtd_text = ""
    text = ""
    extended_text = "no"
    tweet_type = "original"
    reply_userid = None
    reply_screen = None
    reply_statusid = None

    # rt
    rt_qtd_count = 0
    rt_rt_count = 0
    rt_reply_count = 0
    rt_fav_count = 0
    rt_tweetid = None

    # quoted
    qtd_qtd_count = 0
    qtd_rt_count = 0
    qtd_reply_count = 0
    qtd_fav_count = 0
    qtd_tweetid = None

    if "extended_tweet" in status:
        text=status['extended_tweet']['full_text']
        extended_text = "yes"
    elif "text" in status: 
        text = status['text']
        extended_text = "no"

    # take care of retweets
    if 'retweeted_status' in status: 
        rt_tweetid = status['retweeted_status']['id_str']
        if 'extended_tweet' in status['retweeted_status']: 
            rt_text = status['retweeted_status']['extended_tweet']['full_text']
            extended_text = "yes"
            tweet_type = "retweeted_tweet_without_comment"
        elif 'text' in status: 
            rt_text = status['retweeted_status']['text']
            extended_text = "no"
            tweet_type = "retweeted_tweet_without_comment"

        if 'quote_count' in status['retweeted_status']:
            rt_qtd_count = status['retweeted_status']['quote_count']

        if 'retweet_count' in status['retweeted_status']:
            rt_rt_count = status['retweeted_status']['retweet_count']

        if 'reply_count' in status['retweeted_status']:
            rt_reply_count = status['retweeted_status']['reply_count']

        if 'favorite_count' in status['retweeted_status']:
            rt_fav_count = status['retweeted_status']['favorite_count']


    # take care of quoted texts
    if 'quoted_status' in status:
        qtd_tweetid = status['quoted_status']['id_str']
        if 'extended_tweet' in status['quoted_status']:
            qtd_text = status['quoted_status']['extended_tweet']['full_text']
            extended_text = "yes"
            tweet_type = "quoted_tweet"
        elif 'text' in status['quoted_status']:
            qtd_text = status['quoted_status']['text']
            extended_text = "no"
            tweet_type = "quoted_tweet"

        if 'quote_count' in status['quoted_status']:
            qtd_qtd_count = status['quoted_status']['quote_count']

        if 'retweet_count' in status['quoted_status']:
            qtd_rt_count = status['quoted_status']['retweet_count']

        if 'reply_count' in status['quoted_status']:
            qtd_reply_count = status['quoted_status']['reply_count']

        if 'favorite_count' in status['quoted_status']:
            qtd_fav_count = status['quoted_status']['favorite_count']


    if status['in_reply_to_status_id_str'] is not None and not status['truncated']:
        tweet_type = "reply"
        reply_userid = status['in_reply_to_user_id']
        reply_screen = status['in_reply_to_screen_name']
        reply_statusid = status['in_reply_to_status_id']

    elif status['in_reply_to_status_id_str'] is not None and status['truncated']:
        tweet_type = "reply"
        reply_userid = status['in_reply_to_user_id']
        reply_screen = status['in_reply_to_screen_name']
        reply_statusid = status['in_reply_to_status_id']

    return(tweet_type,text.replace("\n", " "),extended_text.replace("\n", " "), rt_text.replace("\n", " "), qtd_text.replace("\n", " "), reply_userid, reply_screen, reply_statusid, 
        rt_qtd_count, rt_rt_count, rt_reply_count, rt_fav_count, rt_tweetid, qtd_qtd_count, qtd_rt_count, 
        qtd_reply_count, qtd_fav_count, qtd_tweetid)

def get_hashtags(status,tweet_type,extended_text):
    rt_screen=''
    rt_userid=''
    q_screen=''
    q_userid=''
    rt_loc = ''
    q_loc = ''

    hashtag = []
    rt_hashtag = []
    q_hashtag = []

    if "extended_tweet" in status: 
        hashtag_list = status["extended_tweet"]["entities"]["hashtags"]
    else: 
        hashtag_list = status["entities"]["hashtags"]

    
    if len(hashtag_list)>=1:
        for i in hashtag_list:
            hashtag.append(i["text"])
    else:
        pass
            

    if tweet_type=="retweeted_tweet_without_comment":
        rt_screen=status['retweeted_status']['user']['screen_name']
        rt_userid=status['retweeted_status']['user']['id_str']
        rt_loc = status['retweeted_status']['user']['location']

        if "extended_tweet" in status["retweeted_status"]: 
            hashtag_list = status["retweeted_status"]["extended_tweet"]["entities"]["hashtags"]
        else: 
            hashtag_list = status["retweeted_status"]["entities"]["hashtags"]

        if len(hashtag_list)>=1:
            for i in hashtag_list:
                rt_hashtag.append(i["text"])
        else:
            pass
            

    elif tweet_type=="quoted_tweet":
        try: 
            q_screen=status['quoted_status']['user']['screen_name']
            q_userid=status['quoted_status']['user']['id_str']
            q_loc = status['quoted_status']['user']['location']
        except: 
            pass
        try:
            if "extended_tweet" in status["quoted_status"]: 
                hashtag_list = status["quoted_status"]["extended_tweet"]["entities"]["hashtags"]
            else: 
                hashtag_list = status["quoted_status"]["entities"]["hashtags"]

            
            if len(hashtag_list)>=1:
                for i in hashtag_list:
                    q_hashtag.append(i["text"])
            else:
                pass
                
        except:
            pass

    return(hashtag,rt_userid,rt_screen, rt_hashtag, rt_loc, q_userid,q_screen, q_hashtag, q_loc)


def get_profile_image(status):
    try:
        profile_pic_url = status['user']['profile_image_url']
        profile_banner_url = status['user']['profile_banner_url']
    except:
        profile_pic_url = ''
        profile_banner_url = ''
    return(profile_pic_url,profile_banner_url)

def main(list_of_dicts):

    fields=['tweetid','userid','screen_name','date','lang','location', "place_id", "place_url", "place_type", \
        "place_name", "place_full_name", "place_country_code", "place_country", "place_bounding_box", 'text','extended','coord', 'reply_userid', 'reply_screen', 'reply_statusid',\
        'tweet_type', "friends_count", "listed_count", "followers_count", "favourites_count", \
        "statuses_count", "verified", "hashtag", 'urls_list','profile_pic_url', 'profile_banner_url',  \
        'display_name', 'date_first_tweet', 'account_creation_date', 'rt_urls_list','mentionid',\
        'mentionsn','rt_screen','rt_userid', 'rt_text', 'rt_hashtag', 'rt_qtd_count', 'rt_rt_count', \
        'rt_reply_count', 'rt_fav_count', 'rt_tweetid', 'rt_location', 'qtd_screen','qtd_userid', 'qtd_text', 'qtd_hashtag', \
        'qtd_qtd_count', 'qtd_rt_count', 'qtd_reply_count', 'qtd_fav_count', 'qtd_tweetid', 'qtd_urls_list', 'qtd_location', 'sent_vader', \
        "token", "media_urls", "rt_media_urls", "q_media_urls"]
    
    data_frame = []

    stop_words = set(stopwords.words('english')) 
    stop_words.add("&amp;")
    stop_words.add("…")

    count = 0
    language_dist = {}
    curr_ids = []
    analyser = SentimentIntensityAnalyzer()
    tknzr = TweetTokenizer(preserve_case=False, reduce_len=False, strip_handles=False)

    for status in list_of_dicts:
                
        tweetid=status['id_str']

        screen_name=status['user']['screen_name']
        userid=status['user']['id_str']
        date=status['created_at']
        lang=status['lang']

        place_id = None
        place_url = None
        place_type = None
        place_name = None
        place_full_name = None
        place_country_code = None
        place_country = None
        place_bounding_box = None
        place = status['place']
        if place is not None: 
            place_id = place['id']
            place_url = place['url']
            place_type = place['place_type']
            place_name = place['name']
            place_full_name = place['full_name']
            place_country_code = place['country_code']
            place_country = place['country']
            place_bounding_box = place['bounding_box']

        location=status['user']['location']
        coord=status['coordinates']
        friends_count=status['user']['friends_count']
        listed_count=status['user']['listed_count']
        followers_count=status['user']['followers_count']
        favourites_count=status['user']['favourites_count']
        statuses_count=status['user']['statuses_count']
        verified=status['user']['verified']
        display_name = status['user']['name']
        date_first_tweet = status['created_at']
        account_creation_date = status['user']['created_at']
        profile_pic_url,profile_banner_url=get_profile_image(status)

        # add quoted and retweet tweet ids 

        tweet_type,text,extended_text, rt_text, qtd_text, reply_userid, reply_screen, \
        reply_statusid, rt_qtd_count, rt_rt_count, rt_reply_count, rt_fav_count, rt_tweetid,\
        qtd_qtd_count, qtd_rt_count, qtd_reply_count, qtd_fav_count, qtd_tweetid  = get_tweet_text(status)

        urls_list, rt_urls_list, qtd_urls_list = get_urls(status,tweet_type,extended_text)
        #rt_urls_list=get_rt_urls(status,tweet_type,extended_text)
        mentionid,mentionsn=get_mentions(status)
        hashtag, rt_userid,rt_screen, rt_hashtag, rt_loc, qtd_userid, qtd_screen, qtd_hashtag, qtd_loc = get_hashtags(status, tweet_type, extended_text) 

        media_urls, rt_media_urls, q_media_urls = get_media_urls(status, tweet_type, extended_text)

        # vader sentiment analysis here
        comp_tweet = "" 
        if tweet_type == "quoted_tweet": 
            # do retweet text first

            try: 
                if "RT @" not in text: 
                    comp_tweet += text 
                else: 
                    pass
            except: 
                pdb.set_trace()

            try: 
                if rt_text: 
                    comp_tweet += (" " + rt_text)

                comp_tweet += (" " + qtd_text)

            except:
                pdb.set_trace()

        elif tweet_type == "retweeted_tweet_without_comment":
            comp_tweet += rt_text


        else: 
            if tweet_type != "original" and tweet_type != "reply":
                print ("something may be wrong here")
                pdb.set_trace()
            comp_tweet += text


        sent_vader = analyser.polarity_scores(comp_tweet)

        # tokenize the comp_tweet

        token_tweet = comp_tweet.replace("RT", "")
        token_tweet = token_tweet.replace("#", "")
        token_tweet = token_tweet.replace("&amp;", "")
        token_tweet = token_tweet.replace("…", "")
        token_tweet = token_tweet.replace (",", " ")
        token_tweet = token_tweet.replace("\n", " ")
        token_tweet = token_tweet.replace("!", "")
        token_tweet = token_tweet.replace(":", "")
        token_tweet = token_tweet.replace("(", " ")
        token_tweet = token_tweet.replace(")", " ")
        tokens = token_tweet.split()

        filtered_tweet = [w for w in tokens if not w in stop_words] 
        tokenized_tweet = []
        for w in filtered_tweet: 
            if "https//t.co" not in w: 
                tokenized_tweet.append(w)
        tokenized_tweet = (' '.join(tokenized_tweet) ).lower()
        tokenized_tweet = tokenized_tweet.replace("/", " ")
        #tokenized_tweet = tokenized_tweet.replace(".", " ")
        tokenized_tweet = tokenized_tweet.replace('"', " ")
        tokenized_tweet = tokenized_tweet.replace('?', " ")
        #tokenized_tweet = tknzr.tokenize(' '.join(tokenized_tweet))

        row=[tweetid,userid,screen_name,date,lang,location, place_id, place_url, place_type, place_name, place_full_name, 
        place_country_code, place_country, place_bounding_box, text,extended_text,coord, reply_userid, reply_screen, reply_statusid, 
        tweet_type,friends_count,listed_count,followers_count,favourites_count,statuses_count,verified,hashtag, urls_list, profile_pic_url,profile_banner_url, 
        display_name, date_first_tweet,account_creation_date,rt_urls_list,mentionid,mentionsn,rt_screen,rt_userid, rt_text, rt_hashtag, rt_qtd_count, rt_rt_count,
            rt_reply_count, rt_fav_count, rt_tweetid, rt_loc, qtd_screen,qtd_userid, qtd_text, qtd_hashtag, 
            qtd_qtd_count, qtd_rt_count, qtd_reply_count, qtd_fav_count, qtd_tweetid ,qtd_urls_list, qtd_loc, sent_vader['compound'], tokenized_tweet, media_urls, rt_media_urls, q_media_urls] 

        data_frame.append(row)

        count +=1

    print(f"Successfully parsed {count} tweets.")
    df = pd.DataFrame(data_frame, columns=fields)
    #print(df)
    return df


from setuptools import setup, find_packages

classifiers = [
	'Operating System :: OS Independent',
	'License :: OSI Approved :: MIT License',
	'Programming Language :: Python :: 3'
	]

setup(
	name='antivaxxtweetanalyzer',
	version='0.0.1',
	description='Identify if a Twitter account displays anti-vaccine sentiment.',
	py_modules=["AvaxModel"],
	#package_dir={'': 'src'},
	author = 'Matheus Schmitz',
	author_email = 'mschmitz@usc.edu',
	packages = find_packages(),
	classifiers = classifiers
	)